
/**
 *  @file     LightNib
 *  @version  0.10
 *
 *  
 *  @note     This program reads from the Rx<-0 pin the binary data coming from the ArduIMU hardware Version 3
 *            Blinks the LED on Arduino UNO for confirmation.
 *  
 *            Based on the work of Francesco Cruciani, Francesco Vitullo
 *            ArduIMU - Arduino binary serial communication
 *            15/02/2011 - www.codesigns.it
 */

/*****************
 * NeoPixel
 *****************/
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define PIN 10 //pin used to drive the neoPixels
// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(8, PIN, NEO_GRB + NEO_KHZ800);

/*****************
 * Toggle buttons
 *****************/
int pinOnOff = 9;
int pinExtra = 8;
int pinIRled1 = 7;
int pinIRled2 = 6;
int pinLed=13;


/*****************
 * ARDUIMU v3
 *****************/
// pin for serial input in arduino
#define RX 0
#define TX 1
// serial rate for arduIMU
#define IMU_SERIAL_RATE 38400
// message tag
#define IMU_TAG "DIYd"
// number of messages send
#define IMU_VALUES 3

// union structure for serial received binary data
typedef union{
  int16_t word;
  byte  valByte[2];
} IMUpacket;

// ArduIMU V3 in binary mode is set to send the following packet:
//    ["DIYd"]  The preamble string. A four letter string, each letter represented with a byte
//    [0x06] [0x02] Dogma bytes. It always sends two fixed bytes right after the "DIYd" string
//    [2 bytes Roll angle] least significant byte first
//    [2 bytes pitch angle] least significant byte first
//    [2 bytes yaw angle] least significant byte first
//    [2 bytes checksum] Finally, the checksum. ArduIMU computes the checksum with all bytes after the preamble string.
//    
//    So, it is necessary to store 8 bytes to compute the checksum and validate the packet received.
//    In technical words, we require an 4 IMUPacket array. Since IMU_VALUES is 3, we just add one.

// intialization of the structures for each passed value
IMUpacket serialPackets[IMU_VALUES+1];



// first tag in message
char imuTag[] = IMU_TAG;
// first tag in message buffer
char IMUtagBuffer[sizeof(imuTag)];
// check for new data
boolean newData = false;
// vars for checksum
byte msg_checksum_a;
byte msg_checksum_b;

// waits for n bytes to receive
void wait_for_bytes(byte number){
  while(Serial.available() < number);
}
// sums all the values and confront the result with the checksum passed
  boolean checksum_is_true(){
  byte current_msg_checksum_a = 0;
  byte current_msg_checksum_b = 0;
  for(uint8_t i=0; i<(IMU_VALUES+1); i++){
    current_msg_checksum_a += serialPackets[i].valByte[0];
    current_msg_checksum_b += current_msg_checksum_a;
                current_msg_checksum_a += serialPackets[i].valByte[1];
    current_msg_checksum_b += current_msg_checksum_a;
  }

  return (current_msg_checksum_a == msg_checksum_a && current_msg_checksum_b == msg_checksum_b);
}



bool readPacket(){
  bool ans = false;
  
  if(Serial.available()>0){
    digitalWrite(pinLed, HIGH);
    // check if the byte is the start of new messange
    wait_for_bytes(1);
    IMUtagBuffer[0] = Serial.read(); //  D
    
    if(IMUtagBuffer[0]==imuTag[0]){
      wait_for_bytes(3);
      IMUtagBuffer[1] = Serial.read(); // I
      IMUtagBuffer[2] = Serial.read(); // Y
      IMUtagBuffer[3] = Serial.read(); // d

      if(IMUtagBuffer[1]==imuTag[1] && IMUtagBuffer[2]==imuTag[2] && IMUtagBuffer[3]==imuTag[3]){
        // New messange has arrived
        newData = true;
        // The preamble string has been received. 
        // Now we have to store in the packet: 
        // the two dogma bytes + IMU_VALUES*2 bytes + two checksum bytes
        // Hence:
        wait_for_bytes((IMU_VALUES*2)+4);
        
        // Store Dogma bytes
        serialPackets[0].valByte[0] = Serial.read();
        serialPackets[0].valByte[1] = Serial.read(); 
        
        //Store all EULER angles coming from the IMU 
        serialPackets[1].valByte[0] = Serial.read();
        serialPackets[1].valByte[1] = Serial.read(); // roll

        serialPackets[2].valByte[0] = Serial.read();
        serialPackets[2].valByte[1] = Serial.read(); // pitch

        serialPackets[3].valByte[0] = Serial.read();
        serialPackets[3].valByte[1] = Serial.read(); // yaw
        
        // Store the checksum
        msg_checksum_a = Serial.read();
        msg_checksum_b = Serial.read();
      }
      else
      {
        // No message to report
        newData=false;
      }
    }
    if (newData && checksum_is_true()){
      ans = true;
    }else{
      msg_checksum_a = 0;
      msg_checksum_b = 0;
    }
  }else{
    digitalWrite(pinLed, LOW);
  }
  return ans;
}


// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

int readInputs(void){
  int readOnOff = !digitalRead(pinOnOff);
  int readExtra = !digitalRead(pinExtra);
  int answer = 0;
  
  if( readOnOff == 1 && readExtra == 0)
    answer = 1;
  else if( readOnOff == 0 && readExtra == 1)
    answer = 2;
  else if( readOnOff == 1 && readExtra == 1)
    answer = 3;
  
  return answer;
}

void setup(){
  //Light Nib related
  pinMode(pinOnOff,INPUT);
  pinMode(pinExtra,INPUT);
  pinMode(pinIRled1,OUTPUT);
  pinMode(pinIRled2,OUTPUT);
  //IMU related initalization 
  Serial.begin(IMU_SERIAL_RATE);
  pinMode(RX, INPUT);
  pinMode(pinLed, OUTPUT);  //Blinking LED for confirmation or data being received from the IMU
  digitalWrite(pinLed, LOW);
  
  //NeoPixel routines
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
}

int rainbowCycle = 0;
void loop(){
  int toc,angle;
  
  if (readPacket()){                        // Read packet from IMU, validate it
    angle = abs(serialPackets[1].word/100); //Get the angle of orientation.
    toc = map(angle, -90, 90, 0, 55);       // scale it to use it with the servo (value between 0 and 180)
    int buttonStatus = readInputs();    
    switch (buttonStatus) {                 // do stuff with received data values only if the button is ON
      case 0:
        colorWipe(strip.Color(0, 0, 0), 0);
        break;
      case 1:
        for(uint16_t i=0; i<strip.numPixels(); i++) {
          strip.setPixelColor(i,strip.Color(toc, 0, 255)); 
          strip.show();
        }
        break;
      case 2:
        rainbowCycle+=1;
        for(uint16_t i=0; i< strip.numPixels(); i++) {
          strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + rainbowCycle) & 255));
        }
        strip.show();
        if (rainbowCycle > 256*5){
          rainbowCycle = 0;
        }
        break;
      case 3:
        for(uint16_t i=0; i<strip.numPixels(); i++) {
          if(i == 2 || i== 3){
            strip.setPixelColor(i,strip.Color(0, 0, 0)); 
          }else{
            strip.setPixelColor(i,strip.Color(255, 255, 255)); 
          }
          strip.show();
        }
        break;      
    }
  } 
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
