

/**
 *  @file   ReadBinaryArduIMUv3.pde
 *  @version  0.20beta
 *
 *  
 *  @note     This program reads from the Rx<-0 pin the binary data coming from the ArduIMU hardware Version 3
 *            Blinks the LED on Arduino UNO for confirmation.
 *  
 *            Based on the work of Francesco Cruciani, Francesco Vitullo
 *            ArduIMU - Arduino binary serial communication
 *            15/02/2011 - www.codesigns.it
 */


// pin for serial input in arduino
#define RX 0
#define TX 1
// serial rate for arduIMU
#define IMU_SERIAL_RATE 38400
// message tag
#define IMU_TAG "DIYd"
// number of messages send
#define IMU_VALUES 3

// union structure for serial received binary data
typedef union{
  int16_t word;
  byte  valByte[2];
} IMUpacket;

// ArduIMU V3 in binary mode is set to send the following packet:
//    ["DIYd"]  The preamble string. A four letter string, each letter represented with a byte
//    [0x06] [0x02] Dogma bytes. It always sends two fixed bytes right after the "DIYd" string
//    [2 bytes Roll angle] least significant byte first
//    [2 bytes pitch angle] least significant byte first
//    [2 bytes yaw angle] least significant byte first
//    [2 bytes checksum] Finally, the checksum. ArduIMU computes the checksum with all bytes after the preamble string.
//    
//    So, it is necessary to store 8 bytes to compute the checksum and validate the packet received.
//    In technical words, we require an 4 IMUPacket array. Since IMU_VALUES is 3, we just add one.

// intialization of the structures for each passed value
IMUpacket serialPackets[IMU_VALUES+1];



// first tag in message
char imuTag[] = IMU_TAG;
// first tag in message buffer
char IMUtagBuffer[sizeof(imuTag)];
// check for new data
boolean newData = false;
// vars for checksum
byte msg_checksum_a;
byte msg_checksum_b;

// waits for n bytes to receive
void wait_for_bytes(byte number){
  while(Serial.available() < number);
}
// sums all the values and confront the result with the checksum passed
boolean checksum_is_true(){
  byte current_msg_checksum_a = 0;
  byte current_msg_checksum_b = 0;
  for(uint8_t i=0; i<(IMU_VALUES+1); i++){
    current_msg_checksum_a += serialPackets[i].valByte[0];
    current_msg_checksum_b += current_msg_checksum_a;
                current_msg_checksum_a += serialPackets[i].valByte[1];
    current_msg_checksum_b += current_msg_checksum_a;
  }

  return (current_msg_checksum_a == msg_checksum_a && current_msg_checksum_b == msg_checksum_b);
}

void setup(){
  Serial.begin(IMU_SERIAL_RATE);
  pinMode(RX, INPUT);
  //Blinking LED for confirmation
  pinMode(13, OUTPUT);
  digitalWrite(13, LOW);
}

bool readPacket(){
  bool ans = false;
  
  if(Serial.available()>0){
    
    // check if the byte is the start of new messange
    wait_for_bytes(1);
    IMUtagBuffer[0] = Serial.read(); //  D
    
    if(IMUtagBuffer[0]==imuTag[0]){
      wait_for_bytes(3);
      IMUtagBuffer[1] = Serial.read(); // I
      IMUtagBuffer[2] = Serial.read(); // Y
      IMUtagBuffer[3] = Serial.read(); // d

      if(IMUtagBuffer[1]==imuTag[1] && IMUtagBuffer[2]==imuTag[2] && IMUtagBuffer[3]==imuTag[3]){
        // New messange has arrived
        newData = true;
        // The preamble string has been received. 
        // Now we have to store in the packet: 
        // the two dogma bytes + IMU_VALUES*2 bytes + two checksum bytes
        // Hence:
        wait_for_bytes((IMU_VALUES*2)+4);
        
        // Store Dogma bytes
        serialPackets[0].valByte[0] = Serial.read();
        serialPackets[0].valByte[1] = Serial.read(); 
        
        //Store all EULER angles coming from the IMU 
        serialPackets[1].valByte[0] = Serial.read();
        serialPackets[1].valByte[1] = Serial.read(); // roll

        serialPackets[2].valByte[0] = Serial.read();
        serialPackets[2].valByte[1] = Serial.read(); // pitch

        serialPackets[3].valByte[0] = Serial.read();
        serialPackets[3].valByte[1] = Serial.read(); // yaw
        
        // Store the checksum
        msg_checksum_a = Serial.read();
        msg_checksum_b = Serial.read();
      }
      else
      {
        // No message to report
        newData=false;
      }
    }
    if (newData && checksum_is_true()){
      ans = true;
    }else{
      msg_checksum_a = 0;
      msg_checksum_b = 0;
    }
  }
  return ans;
}
void blinkLED(){
  digitalWrite(13, HIGH);
  delay(50);
  digitalWrite(13, LOW);
  delay(50);
}
void loop(){
  if (readPacket()){
    // do stuff with received and valid data values
    blinkLED();
  } 
}
